; Installer script for win32 gaim-blogger protocol plugin
; Herman Bloggs <hermanator12002@yahoo.com>

; NOTE: this .NSI script is designed for NSIS v2.0 RC2+

;--------------------------------
;Include Modern UI

  !include "MUI.nsh"

;--------------------------------
;Product Info

  Name "Gaim-Blogger ${VERSION}"

;--------------------------------
;Configuration

  OutFile "gaim-blogger-${VERSION}.exe"
  SetCompressor lzma
  SetDateSave on

;--------------------------------
;Pages
  
  !insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE     "./COPYING"
  !insertmacro MUI_PAGE_INSTFILES

  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES

;--------------------------------
;Modern UI Configuration

  !define MUI_ABORTWARNING
  !define MUI_CUSTOMFUNCTION_GUIINIT checkGaimVersion
;--------------------------------
;Languages
 
  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Defines

!define GAIM_REG_KEY            "SOFTWARE\gaim"
!define BLOGGER_UNINSTALLER     "gaim-blogger-uninst.exe"

;--------------------------------
;Main Install Section

Section "Dummy Section" SecGaimBlogger
  SectionIn 1 RO

  ReadRegStr $R0 HKLM ${GAIM_REG_KEY} ""
  StrCmp $R0 "" 0 next1
    ReadRegStr $R0 HKCU ${GAIM_REG_KEY} ""
      StrCmp $R0 "" 0 next1
        MessageBox MB_OK "Gaim was not found on this machine." IDOK
        Quit

  next1:
    IfFileExists "$R0\*.*" 0 no_dir
      Call CheckUserInstallRights
      Pop $R1
      StrCmp $R1 "HKLM" 0 next2
        SetShellVarContext "all"
      next2:
      ; Gaim files
      StrCpy $INSTDIR $R0
      SetOverwrite on
      SetOutPath $R0
      File ..\win32-dev\expat\bin\libexpat.dll
      SetOutPath "$R0\plugins"
      File .\libblogger.dll
      SetOutPath "$R0\pixmaps\gaim\status\default"
      File .\pixmaps\blogger.png
      CreateShortCut "$SMPROGRAMS\Gaim\Blogger-Uninstall.lnk" "$INSTDIR\${BLOGGER_UNINSTALLER}"
      SetOverwrite off
      Goto done

  no_dir:
    MessageBox MB_OK "Gaim registry entry found for invalid directory path: $R0" IDOK
    Quit

  done:
    ; write out uninstaller
    SetOverwrite on
    WriteUninstaller "$INSTDIR\${BLOGGER_UNINSTALLER}"
    SetOverwrite off

SectionEnd ; end of default Gaim section

Section "Uninstall"
  Call un.CheckUserInstallRights
  Pop $R0
  StrCmp $R0 "HKLM" 0 cont_uninst
    SetShellVarContext "all"

  cont_uninst:
  Delete "$INSTDIR\libexpat.dll"
  Delete "$INSTDIR\plugins\libblogger.dll"
  Delete "$INSTDIR\pixmaps\gaim\status\default"
  Delete "$INSTDIR\${BLOGGER_UNINSTALLER}"
  RMDir "$INSTDIR"

  ; Shortcuts..
  Delete "$SMPROGRAMS\Gaim\Blogger-Uninstall.lnk"
  RMDir "$SMPROGRAMS\Gaim"
SectionEnd ; end of uninstall section

LangString NO_GAIM_VERSION ${LANG_ENGLISH} "Unable to determine installed Gaim version."
LangString BAD_GAIM_VERSION_1 ${LANG_ENGLISH} "Incompatible version.$\r$\n$\r$\nThis version of the Gaim-Blogger plugin was built for Gaim version ${VERSION}.  It appears that you have Gaim version"
LangString BAD_GAIM_VERSION_2 ${LANG_ENGLISH} "installed.$\r$\n$\r$\nSee http://gaim-blogger.sourceforge.net for more information."

Function checkGaimVersion
  Push $R0
  Push $R1
  Push $R2

  ClearErrors
  ReadRegStr $R0 HKLM "SOFTWARE\gaim" "version"
  IfErrors readHKCUGaimVersion compareVersion

  readHKCUGaimVersion:
  ReadRegStr $R0 HKCU "SOFTWARE\gaim" "version"
  IfErrors noGaimInstallFound compareVersion

  noGaimInstallFound:
    MessageBox MB_OK|MB_ICONSTOP "$(NO_GAIM_VERSION)"
    Quit

  compareVersion:
  StrCmp $R0 ${VERSION} GaimVersionOK
    ; Versions are in the form of X.Y.Z
    ; If X is different, then we shouldn't install
    Push $R0
    Call GetMajorVersion
    IfErrors BadVersion
    Pop $R2
    Push ${VERSION}
    Call GetMajorVersion
    IfErrors noGaimInstallFound
    Pop $R1
    ;Check that both version's X is the same
    IntCmp $R1 $R2 GaimVersionOK
    BadVersion:
    MessageBox MB_OK|MB_ICONSTOP "$(BAD_GAIM_VERSION_1) $R0 $(BAD_GAIM_VERSION_2)"
    Quit
  GaimVersionOK:
  Pop $R2
  Pop $R1
  Pop $R0
FunctionEnd

; Extract the part of a string prior to "." (or the whole string if there is no ".")
; If no "." was found, the ErrorFlag will be set
Function GetMajorVersion
    ClearErrors
    Pop $0
    StrCpy $1 "0"

    startGetMajorVersionLoop:
    ;avoid an infinite loop, if we have gotten the whole initial string, exit the loop and set the error flag
    StrCmp $2 $0 GetMajorVersionSetErrorFlag
    IntOp $1 $1 + 1
    ;Append the next character in $0 to $2
    StrCpy $2 $0 $1
    ;store the next character in $3
    StrCpy $3 $0 1 $1
    ;if the next character is ".", $2 will contain the string prior to "."
    StrCmp $3 "." endGetMajorVersion startGetMajorVersionLoop

    GetMajorVersionSetErrorFlag:
    SetErrors

    endGetMajorVersion:

    Push $2
FunctionEnd


;--------------------------------
;Functions

Function un.CheckUserInstallRights
    ClearErrors
    UserInfo::GetName
    IfErrors Win9x
    Pop $0
    UserInfo::GetAccountType
    Pop $1

    StrCmp $1 "Admin" 0 +3
                StrCpy $1 "HKLM"
        Goto done
    StrCmp $1 "Power" 0 +3
                StrCpy $1 "HKLM"
        Goto done
    StrCmp $1 "User" 0 +3
        StrCpy $1 "HKCU"
        Goto done
    StrCmp $1 "Guest" 0 +3
        StrCpy $1 "NONE"
        Goto done
    ; Unknown error
    StrCpy $1 "NONE"
        Goto done

    Win9x:
        StrCpy $1 "HKLM"

    done:
        Push $1
FunctionEnd

Function CheckUserInstallRights
    ClearErrors
    UserInfo::GetName
    IfErrors Win9x
    Pop $0
    UserInfo::GetAccountType
    Pop $1

    StrCmp $1 "Admin" 0 +3
                StrCpy $1 "HKLM"
        Goto done
    StrCmp $1 "Power" 0 +3
                StrCpy $1 "HKLM"
        Goto done
    StrCmp $1 "User" 0 +3
        StrCpy $1 "HKCU"
        Goto done
    StrCmp $1 "Guest" 0 +3
        StrCpy $1 "NONE"
        Goto done
    ; Unknown error
    StrCpy $1 "NONE"
        Goto done

    Win9x:
        StrCpy $1 "HKLM"

    done:
        Push $1
FunctionEnd
