/*
 * gaim - Blogger (xml-rpc) Protocol Plugin API
 *
 * Copyright (C) 2004, Daniel Atallah <daniel_atallah@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _BLOGGER_H_
#define _BLOGGER_H_

#include "gaim.h"

#if 0
typedef struct _BloggerEditor BloggerEditor;
typedef struct _BloggerEditorUiOps BloggerEditorUiOps;
typedef struct _PostSelector PostSelector;
#endif

typedef struct _BlogPost BlogPost;

typedef struct _UserBlog UserBlog;

typedef struct _BloggerRequest BloggerRequest;

typedef struct _BloggerData BloggerData;

typedef enum _BloggerReq {
	none,
	newPost,
	editPost,
	deletePost,
	getRecentPosts,
	getUsersBlogs,
	getUserInfo,
	getTemplate,
	setTemplate
} BloggerReq;

typedef enum _PendingOps {
	NOOP                           = 0,      /* no pending operation */
	GET_USERS_BLOGS_STARTUP        = 1 << 0, /* For users_blogs_cb */
	GET_USERS_BLOGS_PERIODIC       = 1 << 1, /* For users_blogs_cb */
	GET_POSTS_FOR_EDIT             = 1 << 2, /* For recent_posts_cb */
	GET_POSTS_LAST_POSTED          = 1 << 3, /* For recent_posts_cb */
	GET_POSTS_CHECK_FOR_NEW_POST   = 1 << 4  /* For recent_posts_cb */
} PendingOps;

typedef enum _BlogStatus {
	BLOGGER_NORMAL                 = 1 << 0,
	BLOGGER_CHANGED                = 1 << 1,          /* new post, edited post, template change */
	BLOGGER_NEW_POST               = 1 << 2           /* new post (in case you didn't guess)*/
} BlogStatus;

struct _BlogPost {
	char *postid;
	char *lastModified;
	char *status;
	char *userid;
	char *content;
	char *authorName;
	char *title;
	char *dateCreated;
	char *postDate;
	char *url;
	UserBlog *blog;
};

struct _UserBlog {
	gboolean isAdmin;
	char url[256];
	char blogid[25];
	char lastPublished[25];
	gboolean showTitle;
	char blogName[128];
	BlogStatus blog_status;
	char our_last_postid[25];
	char last_postid[25];
	char last_postid_date[15];
	GaimBuddy *buddy;
};

struct _BloggerRequest {
	BloggerReq pend;
	char *request;
	PendingOps options;
	UserBlog *blog;
};

struct _BloggerData {
	BloggerReq pending;    	/* Pending Blogger Method Request. */
	PendingOps pending_ops; /* More detailed processing info for the method callbacks */
	UserBlog *pending_blog; /* For blog specific requests.. */
	void *cb_data;          /* Set by methods for use by callbacks */
	char* send_data;        /* Method data to post once connected */
	unsigned int sd_length; /* Method data length */
	GQueue* request_queue;  /* If a request is pending, subsequent requests are queued */
	GList* blog_list;       /* List of Blogs for this user */
	gint update_timeout;    /* timeout handle for blogger_update_check_cb */
	GaimConnection *gc;     /* Gaim connection for this blog user */
};

#if 0
/**
 * The Blogger Editor
 */
struct _BloggerEditor {
	char *title;
	char *subtitle;
	char *b1_label; 
	char *b2_label;
	void *b1_cb; 
	void *b2_cb;
	char *editor_text;
	gboolean modal;
	BloggerEditorUiOps *ui_ops;
	
	void *ui_data;
};



/**
 * The Post Selector
 */
struct _PostSelector {

};

/**
 * Blogger UI Operations
 *
 * Any UI representing a blogger editor must assign a filled out BloggerEditorUiOps to the BloggerEditor
 */
struct _BloggerEditorUiOps
{
	void (*show_editor)(BloggerEditor *editor, void *data);
	void (*destroy)(BloggerEditor *editor);
	
				
};
#endif

#endif /* _BLOGGER_H_ */
