/*
 * gaim - Blogger (xml-rpc) Protocol Plugin
 *
 * Copyright (C) 2003, Herman Bloggs <hermanator12002@yahoo.com>
 * GPL Software Supported by www.Madster.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <glib.h>
#include "stack.h"

#define debug_printf printf

stack *stack_new() {
	stack *s = g_new0(struct _stack, 1);
	if(!s)
		return NULL;
	s->top = NULL;
	return s;
}

void stack_push( stack *s, void *data ) {
	stack_item *s_item;
	
	if(!s) {
		debug_printf("stack_push: Received NULL stack pointer.\n");
		return;
	}
	if(!data) {
		debug_printf("stack_push: Received NULL data pointer.\n");
		return;
	}

	s_item = g_malloc(sizeof(stack_item));

	if(!s_item)
		return;

	s_item->data = data;
	s_item->next = s->top;
	s->top = s_item;
}

void *stack_pop( stack *s ) {
	stack_item *s_item;
	void *data;

	if(!s) {
		debug_printf("stack_pop: Received NULL stack pointer.\n");
		return NULL;
	}
	if(!s->top)
		return NULL;
	s_item = s->top;
	data = s_item->data;
	s->top = s_item->next;
	g_free(s_item);
	return data;
}

void *stack_top( stack *s ) {
	if(!s) {
		debug_printf("stack_top: Received NULL stack pointer.\n");
		return NULL;
	}
	if(!s->top)
		return NULL;

	return s->top->data;
}

int stack_empty(stack *s) {
	if(!s) {
		debug_printf("stack_empty: Received NULL stack pointer.\n");
		return 1;
	}
	return (s->top) ? 0 : 1;
}

void stack_free(stack *s) {
	if(!s)
		return;
	if(s->top)
		debug_printf("stack_free: Stack not empty\n");
	else
		g_free(s);
}
