/*
 * gaim - Blogger Protocol Plugin - Blogger xmlrpc http routines
 *
 * Copyright (C) 2003, Herman Bloggs <hermanator12002@yahoo.com>
 * GPL Software Supported by www.Madster.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "internal.h"
#include "xmlrpc_http.h"
#include "debug.h"

#ifdef _WIN32
#include "win32dep.h"
#endif

#define CRLF                 "\r\n"
#define HTTP_VERSION         "HTTP/1.0"


int xmlrpc_http_post(gint fd,
		     const char* useragent,
		     const char* cont_type,
		     const char* uri, 
		     const char* content, 
		     unsigned int length) {
	char *header;
	char *buf, *sbuf, *tmp;
	char *host;
	char *rel_path;
	int slen=0;

	if(!fd) {
		gaim_debug(GAIM_DEBUG_WARNING, "blogger", "xmlrpc_http_post: Invalid socket descriptor\n");
		return 0;
	}
	if(!useragent || !cont_type || !uri || !content) {
		gaim_debug(GAIM_DEBUG_WARNING, "blogger", "xmlrpc_http_post: Received unexpected NULL arguement.\n");
		return 0;
	}

	buf = tmp = g_strdup(uri);
	if((buf=strstr(buf, "http://")))
		buf+=7;
	else if((buf=tmp) && (buf=strstr(buf, "HTTP://")))
		buf+=7;
	else
		buf=tmp;

	host = strtok(buf, "/");
	rel_path = strtok(NULL, "");
	
	/* Build Header */
	header = g_strdup_printf("POST /%s %s" CRLF
				 "User-Agent: %s" CRLF
				 "Host: %s" CRLF
				 "Content-Type: %s" CRLF
				 "Content-length: %u" CRLF
				 CRLF,
				 rel_path ? rel_path : "", HTTP_VERSION,
				 useragent,
				 host ? host : uri,
				 cont_type,
				 length);
	g_free(buf);

	/* Build request */
	slen = strlen(header) + length;
	sbuf = g_malloc0(slen + 1);
	memcpy(sbuf, header, strlen(header));
	memcpy(sbuf + strlen(header), content, length);
	sbuf[slen] = '\0';
	g_free(header);

	if(!write(fd, sbuf, slen)) {
		gaim_debug(GAIM_DEBUG_WARNING, "blogger", "xmlrpc_http_post: Error writing fd.\n");
		g_free(sbuf);
		return 0;
	}
	g_free(sbuf);
	return 1;
}

char* xmlrpc_get_http_post_response(const char *buf, int *len, char **errmsg) {
	const char* header;
	char* ret;
	char *tmp, *tmp1;

	if(!buf)
		return NULL;

	header=buf;
	ret = strstr(buf, "\r\n\r\n");
	if(ret) {
		ret[0] = '\0';
		ret+=4;
	}
	else {
		gaim_debug(GAIM_DEBUG_WARNING, "blogger", "xmlrpc_get_http_post_response: Failed\n");
		return NULL;
	}
	/* match "HTTP/1.X 200" */
	if ((tmp = strstr(header, "HTTP")) && 
	   strncmp(tmp+9, "200", 3) == 0) {
		/*Good http response */
		tmp=strstr(header, "Content-Type: ");
		if(tmp && (strstr(tmp, "text/xml") || strstr(tmp, "application/xml"))) {
			tmp = strstr(header, "Content-Length: ");
			/* Get content length */
			if(tmp && (tmp=strchr(tmp, ' '))) {
				tmp += 1;
				tmp1 = g_strndup(tmp, (int)(strstr(tmp, CRLF) - tmp));
				*len = atoi(tmp1);
				g_free(tmp1);
				return ret;
			}
		}

	}
	/* match "HTTP/1.X 302"  (Redirect)*/
	else if ((tmp = strstr(header, "HTTP")) && 
	   strncmp(tmp+9, "302", 3) == 0) {
		tmp = strstr(tmp, "Location: ");
		if (tmp && (tmp = strstr(tmp, " http://"))) {
			tmp += 8;
			tmp1 = g_strndup(tmp, (int)(strstr(tmp, CRLF) - tmp));
			gaim_debug(GAIM_DEBUG_WARNING, "blogger", "xmlrpc_get_http_post_response: redirect request from server:\n%s\n", tmp1);
			*errmsg = g_strdup_printf(_("The server address has changed to \"%s\", please update your account settings."), tmp1);
			g_free(tmp1);
			return NULL;
		}

	}
	gaim_debug(GAIM_DEBUG_WARNING, "blogger", "xmlrpc_get_http_post_response: Failed to parse HTTP header:\n%s\n", header);
	return NULL;
}

